# -*- coding: utf-8 -*-
"""
Created on Fri Feb 20 16:40:15 2015

@author: mshahbazi
"""
import numpy as np
from DBSLIP import *
from DBSLIP_Approx import *
import matplotlib.pyplot as plt

def Ess(x, origin):
    v2 = (x[2]**2 + x[3]**2)
    l = np.sqrt((x[0] - origin)**2 + x[1]**2)
    return mh*g0*x[1] + 1/2.*mh*v2 + 1/2.*k0*(L0 - l)**2

dt = .005 #nominal integration step size (may vary in the integrator)

# Some initialization: 
g0 = 9.81 #gravity
L0 = 1.0  # spring rest length
mh = 80. # point mass


c1 = 0. # fore foot
c2 = -.1 # rear foot, a fake number for initialization!
yd0 = 0. # initial vertical velocity
x0 = 0. # initial horizontal position

y0 = .98 # initial vertical position
dx0 = 1.2 # initial horizontal velocity

k0 = 16.5e3 # spring stiffness

alpha0 = np.deg2rad(70) # touchdown angle

b = 800 # damping coefficient

kisi = b/(2*np.sqrt(k0*mh)) # damping ratio

print r'$\xi$ = ', kisi

#This is to avoid divided by zero situation:
if kisi<1e-6: b=0

# Exact (numerical) solution:
t1 = np.array([0]) # initiating time vector
x1 = np.array([[0,y0,dx0,0]]) # initial state
xdes1 = x1[-1, :] # desired state (when control authority is of desire)
q1 = np.array([[0,mh,L0,k0,g0,alpha0,c1,c2,b]]) # initial discrete states and ...
s = DBSLIP(t1, x1, q1, xdes1, dt) # instantiating the slip model
#running exact SLIP model for one step
while (s.event!='v' or not s.t[-1][0]) and s.q[-1, 0]!=-1: s()

# Approximate solution:
# first single-stance (SS) phase
t, x, y, dx, dy = DBSLIP_Approx(x1[-1, :], q1[-1, :])
plt.plot(x, y, '--', lw = 2)
q1[-1, 7] = x[-1] + q1[-1, 2]*np.cos(q1[-1, 5])
q1[-1, 0] = 1 # changing mode to double-stance

EE1 = Ess([x[-1], y[-1], dx[-1], dy[-1]], q1[-1, 6])

# double-stance (DS) phase
t, x, y, dx, dy = DBSLIP_Approx([x[-1], y[-1], dx[-1], dy[-1]],\
q1[-1, :])
plt.plot(x, y, '--', lw = 2)
q1[-1, 0] = 3 # changing mode to single-stance


EE2 = Ess([x[-1], y[-1], dx[-1], dy[-1]], q1[-1, 7])
print x[-1], y[-1], dx[-1], dy[-1]

# second single-stance (SS) phase
t, x, y, dx, dy = DBSLIP_Approx([x[-1], y[-1], dx[-1], dy[-1]], \
q1[-1, :])
plt.plot(x, y, '--', lw = 2)

plt.plot(s.x[:, 0], s.x[:, 1], 'k')
plt.plot(s.xt[0], s.xt[1], '*')
plt.plot(s.xl[0], s.xl[1], 'x')

plt.legend(['First SS Approx.', 'DS Approx.',\
'Second SS Approx.', '"Exact" (numerical)', \
'Touchdown', 'Liftoff'], loc='lower right')
plt.xlabel('Horizontal Position, x (m)')
plt.ylabel('Vertical Position, y (m)')

plt.ylim([0, 1])
